@@ -1,5 +1,6 @@
 package Inline::C;
-$VERSION = '0.48';
+$Inline::C::VERSION = '0.52';
+$Inline::C::VERSION = eval $Inline::C::VERSION;
 
 use strict;
 require Inline;
@@ -38,12 +39,13 @@ END
 sub validate {
     my $o = shift;
 
+    print STDERR "validate Stage\n" if $o->{CONFIG}{BUILD_NOISY};
     $o->{ILSM} ||= {};
     $o->{ILSM}{XS} ||= {};
     $o->{ILSM}{MAKEFILE} ||= {};
     if (not $o->UNTAINT) {
 	require FindBin;
-	$o->{ILSM}{MAKEFILE}{INC} = "-I$FindBin::Bin";
+	$o->{ILSM}{MAKEFILE}{INC} = "-I\"$FindBin::Bin\"";
     }
     $o->{ILSM}{AUTOWRAP} = 0 if not defined $o->{ILSM}{AUTOWRAP};
     $o->{ILSM}{XSMODE} = 0 if not defined $o->{ILSM}{XSMODE};
@@ -63,6 +65,19 @@ END
 
     while (@_) {
 	my ($key, $value) = (shift, shift);
+      if ($key eq 'PRE_HEAD') {
+         unless( -f $value) {
+           $o->{ILSM}{AUTO_INCLUDE} = $value . "\n" . $o->{ILSM}{AUTO_INCLUDE};
+         }
+         else {
+           my $insert;
+           open RD, '<', $value or die "Couldn't open $value for reading: $!";
+           while(<RD>) {$insert .= $_}
+           close RD or die "Couldn't close $value after reading: $!";
+           $o->{ILSM}{AUTO_INCLUDE} = $insert . "\n" . $o->{ILSM}{AUTO_INCLUDE};
+         }
+         next;
+      }
 	if ($key eq 'MAKE' or
 	    $key eq 'AUTOWRAP' or
             $key eq 'XSMODE'
@@ -79,14 +94,21 @@ END
 	    $o->add_list($o->{ILSM}{MAKEFILE}, $key, $value, []);
 	    next;
 	}
-	if ($key eq 'INC' or
-	    $key eq 'MYEXTLIB' or
+	if ($key eq 'INC') {
+	    $o->add_string($o->{ILSM}{MAKEFILE}, $key, quote_space($value), '');
+	    next;
+	}
+	if ($key eq 'MYEXTLIB' or
 	    $key eq 'OPTIMIZE' or
 	    $key eq 'CCFLAGS' or
 	    $key eq 'LDDLFLAGS') {
 	    $o->add_string($o->{ILSM}{MAKEFILE}, $key, $value, '');
 	    next;
 	}
+      if ($key eq 'CCFLAGSEX') {
+	    $o->add_string($o->{ILSM}{MAKEFILE}, 'CCFLAGS', $Config{ccflags} . ' ' . $value, '');
+          next;
+      }
 	if ($key eq 'TYPEMAPS') {
           unless(ref($value) eq 'ARRAY') {
 	      croak "TYPEMAPS file '$value' not found"
@@ -94,14 +116,14 @@ END
 	      $value = File::Spec->rel2abs($value);
           }
           else {
-          for (my $i = 0; $i < scalar(@$value); $i++) {
+            for (my $i = 0; $i < scalar(@$value); $i++) {
 	      croak "TYPEMAPS file '${$value}[$i]' not found"
 	        unless -f ${$value}[$i];
-            ${$value}[$i] = File::Spec->rel2abs(${$value}[$i]);
+              ${$value}[$i] = File::Spec->rel2abs(${$value}[$i]);
             }
           }
-	    $o->add_list($o->{ILSM}{MAKEFILE}, $key, $value, []);
-	    next;
+	  $o->add_list($o->{ILSM}{MAKEFILE}, $key, $value, []);
+	  next;
 	}
 	if ($key eq 'AUTO_INCLUDE') {
 	    $o->add_text($o->{ILSM}, $key, $value, '');
@@ -338,6 +360,7 @@ sub get_parser {
 sub get_maps {
     my $o = shift;
 
+    print STDERR "get_maps Stage\n" if $o->{CONFIG}{BUILD_NOISY};
     my $typemap = '';
     my $file;
     $file = File::Spec->catfile($Config::Config{installprivlib},"ExtUtils","typemap");
@@ -353,7 +376,9 @@ sub get_maps {
     if (not $o->UNTAINT) {
 	require FindBin;
 	$file = File::Spec->catfile($FindBin::Bin,"typemap");
-	push(@{$o->{ILSM}{MAKEFILE}{TYPEMAPS}}, $file) if -f $file;
+        if ( -f $file ) {
+	   push(@{$o->{ILSM}{MAKEFILE}{TYPEMAPS}}, $file);
+        }
     }
 }
 
@@ -691,8 +716,10 @@ END
 sub write_Makefile_PL {
     my $o = shift;
     $o->{ILSM}{xsubppargs} = '';
+    my $i = 0;
     for (@{$o->{ILSM}{MAKEFILE}{TYPEMAPS}}) {
-	$o->{ILSM}{xsubppargs} .= "-typemap $_ ";
+	$o->{ILSM}{xsubppargs} .= "-typemap \"$_\" ";
+        $o->{ILSM}{MAKEFILE}{TYPEMAPS}->[$i++] = fix_space($_);
     }
 
     my %options = (
@@ -734,10 +761,15 @@ sub compile {
     ($cwd) = $cwd =~ /(.*)/ if $o->UNTAINT;
 
     chdir $build_dir;
-    $o->call('makefile_pl', '"perl Makefile.PL"', 2);
-    $o->call('make', '"make"', 2);
-    $o->call('make_install', '"make install"', 2);
+    # Run these in an eval block, so that we get to chdir back to
+    # $cwd if there's a failure. (Ticket #81375.)
+    eval {
+      $o->call('makefile_pl', '"perl Makefile.PL"', 2);
+      $o->call('make', '"make"', 2);
+      $o->call('make_install', '"make install"', 2);
+    };
     chdir $cwd;
+    die if $@; #Die now that we've done the chdir back to $cwd. (#81375)
     $o->call('cleanup', 'Cleaning Up', 2);
 }
 
@@ -847,7 +879,9 @@ sub fix_make {
 	if (/^(\w+)\s*=\s*\S+.*$/ and
 	    $fix = $fixes{$1}
 	   ) {
-	    print MAKEFILE "$1 = $o->{ILSM}{$fix}\n"
+	    my $fixed = $o->{ILSM}{$fix};
+	    $fixed = fix_space($fixed) if $fix eq 'install_lib';
+	    print MAKEFILE "$1 = $fixed\n";
 	}
 	else {
 	    print MAKEFILE;
@@ -856,6 +890,83 @@ sub fix_make {
     close MAKEFILE;
 }
 
+sub quote_space {
+
+  # Do nothing if $ENV{NO_INSANE_DIRNAMES} is set
+  return $_[0] if $ENV{NO_INSANE_DIRNAMES};
+
+  # If $_[0] contains one or more doublequote characters, assume
+  # that whitespace has already been quoted as required. Hence,
+  # do nothing other than immediately return $_[0] as is.
+  # We currently don't properly handle tabs either, so we'll
+  # do the same if $_[0] =~ /\t/.
+  return $_[0] if ($_[0] =~ /"/ || $_[0] =~ /\t/);
+
+  # We want to split on /\s\-I/ not /\-I/
+  my @in = split /\s\-I/, $_[0];
+  my $s = @in - 1;
+  my %s;
+  my %q;
+
+  # First up, let's reinstate the ' ' characters that split
+  # removed
+  for(my $i = 0; $i < $s; $i++) {
+    $in[$i] .= ' ';
+  }
+
+  # This for{} block dies if it finds that any of the ' -I'
+  # occurrences in $_[0] are part of a directory name.
+  for(my $i = 1; $i < $s; $i++) {
+    my $t = $in[$i + 1];
+    while($t =~ /\s$/) {chop $t}
+    die "Found a '", $in[$i], "-I", $t, "' directory.",
+        " INC Config argument is ambiguous.",
+        " Please use doublequotes to signify your intentions"
+      if -d ($in[$i] . "-I" . $t);
+  }
+
+  $s++; # Now the same as scalar(@in)
+
+  # Remove (but also Keep track of the amount of) whitespace
+  # at the end of each element of @in.
+  for(my $i = 0; $i < $s; $i++) {
+    my $count = 0;
+    while($in[$i] =~ /\s$/) {
+      chop $in[$i];
+      $count++;
+    }
+    $s{$i} = $count;
+  }
+
+  # Note which elements of @in still contain whitespace. These
+  # (and only these) elements will be quoted
+  for(my $i = 0; $i < $s; $i++) {
+    $q{$i} = 1 if $in[$i] =~ /\s/;
+  }
+
+  # Reinstate the occurrences of '-I' that were removed by split(),
+  # insert any quotes that are needed, reinstate the whitespace
+  # that was removed earlier, then join() the array back together
+  # again.
+  for(my $i = 0; $i < $s; $i++) {
+    $in[$i] = '-I' . $in[$i] if $i;
+    $in[$i] = '"' . $in[$i] . '"' if $q{$i};
+    $in[$i] .= ' ' x $s{$i};
+  }
+
+  # Note: If there was no whitespace that needed quoting, the
+  # original argument should not have changed in any way.
+
+  my $out = join '', @in;
+  $out =~ s/"\-I\s+\//"\-I\//g;
+  $_[0] = $out;
+}
+
+sub fix_space {
+    $_[0] =~ s/ /\\ /g if $_[0] =~ / /;
+    $_[0];
+}
+
 #==============================================================================
 # This routine used by C/t/09parser to test that the expected parser is in use
 #==============================================================================
@@ -868,7 +979,6 @@ sub _parser_test {
     }
 
     warn $! if !open(TEST_FH, '>>', "$dir/parser_id");
-
     print TEST_FH $_[0];
     warn $! if !close(TEST_FH);
 }
